/******************************************************************************
* pathDFS.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * pathDFS.h
 *
 *  Created on: Jul 1, 2010
 *      Author: pajor
 */

#ifndef PATHDFS_H_
#define PATHDFS_H_

#include "../dataStructures/graph.h"
#include "../dataStructures/path.h"

#include <boost/graph/graph_traits.hpp>


namespace schematization {


template<typename graphType>
class pathDFS {

public:
	typedef typename graphType::vertex_descriptor vertex_descriptor;
	typedef typename graphType::edge_descriptor edge_descriptor;

	typedef vector<vertex_descriptor> verticesType;

public:

	pathDFS(graphType &g) : graph(g) {
	}


	// This performs a DFS beginning at some edge e
	void run(edge_descriptor source_e) {
		p.clear();
		p_graph.clear();

		p.name = graph[source_e].name;
		p.category = graph[source_e].category;


		// Initialize stack with target node of source_e
		vertex_descriptor u = source(source_e, graph);
		add_node(u);
		source_node = u;
		vertex_descriptor v = target(source_e, graph);


		while(!graph[v].important) {
			// Add the node v to the path
			add_node(v);
			p_graph.push_back(v);

			// Get first outgoing edge (there must be only one)
			edge_descriptor e = get_first_out_edge(v, graph);

			// Traverse the path
			v = target(e, graph);
		}

		add_node(v);
		target_node = v;
	}


	// This method retrieves the path
	path &get_path()  {
		return p;
	}


	// Get the first node on the path (source-node)
	vertex_descriptor get_source_node() const {
		return source_node;
	}
	// Get the last node on the path (target-node)
	vertex_descriptor get_target_node() const {
		return target_node;
	}

	// This method deletes the path from graph ;-)
	void delete_path() {
		for (size_t n = 0; n < p_graph.size(); ++n) {
			clear_vertex(p_graph[n], graph);
			remove_vertex(p_graph[n], graph);
		}
	}

protected:

	// This adds a node to the path
	inline void add_node(const vertex_descriptor &v) {
		path_node n(graph[v].coord);
		p.push_node(n);
	}

private:

	// This is the graph we are operating on
	graphType &graph;

	// These are the source and target nodes of the path
	vertex_descriptor source_node, target_node;

	// This is the path generated from the search
	path p;

	// This vector holds the *INNER* vertices which correspond to the path in the graph
	// They must be deleted and substituted by the *INNER* vertices of the (simplified)
	// path
	verticesType p_graph;
};


}

#endif /* PATHDFS_H_ */
