/******************************************************************************
* ramerDouglasPeucker.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * ramerDouglasPeucker.h
 *
 *  Created on: Jul 1, 2010
 *      Author: pajor
 */

#ifndef RAMERDOUGLASPEUCKER_H_
#define RAMERDOUGLASPEUCKER_H_


#include "../dataStructures/graph.h"
#include "../geometry/tools.h"

namespace schematization {

template<typename pathType>
class ramer_douglas_peucker {

public:

	// Construct the class here
	ramer_douglas_peucker(pathType &p) : input_path(p) {

	}


	// Run the algorithm
	void run(const double eps) {
		simplified_path = input_path;

		// Initialize the erase flag (set to true for every nodes). Nodes
		// that should be kept are identified during the algorithm
		for (size_t n = 0; n < simplified_path.size(); ++n)
			simplified_path[n].to_erase = true;


		// RDG-recursion
		ramer_douglas_peucker_rec(0, simplified_path.size() - 1, eps);

		// Reduction
		simplified_path.reduce();
	}


	// This retrieves the simplified path
	pathType &get_simplified_path() {
		return simplified_path;
	}


protected:

	// This is the recursive subrouine of the rdg algorithm
	void ramer_douglas_peucker_rec(const size_t lower_index, const size_t upper_index, const double eps) {
		// Set start and end nodes as important (they should be kept)
		simplified_path[lower_index].to_erase = false;
		simplified_path[upper_index].to_erase = false;

		// If the size of the path is smaller or equal 2 points, there is nothing to simplify.
		if (upper_index - lower_index < 2)
			return;

		// Find point with largest distance.
		double max_dist = 0.0;
		int max_dist_index = -1;
		for (size_t p = lower_index + 1; p < upper_index; ++p) {
			simplified_path[p].to_erase = true;
			double dist_n = tools::dist_3p(simplified_path[lower_index].coord, simplified_path[upper_index].coord, simplified_path[p].coord);
			if (dist_n > max_dist) {
				max_dist = dist_n;
				max_dist_index = p;
			}
		}
		//assert(max_dist_index != -1); bullshit
		if (max_dist_index == -1)
			return;

		// Call recursively, if and only if max_dist > eps
		if (max_dist > eps) {
			simplified_path[max_dist_index].to_erase = false; // Mark the node as important
			ramer_douglas_peucker_rec(lower_index, max_dist_index, eps);
			ramer_douglas_peucker_rec(max_dist_index, upper_index, eps);
		}
	}
private:

	// This is the path we are operating on
	pathType &input_path;

	// This is the simplified path
	pathType simplified_path;
};




}

#endif /* RAMERDOUGLASPEUCKER_H_ */
