/******************************************************************************
* pfadDynProg.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * pfadDynProg.h
 *
 *  Created on: 24.08.2009
 *      Author: andreas
 */

#ifndef PFADDYNPROG_H_
#define PFADDYNPROG_H_

#include <iostream>
#include <limits>
#include <algorithm>
#include <queue>

#include <gurobi_c++.h>
#include "boost/lexical_cast.hpp"
#include "../types.h"


#define COS_DEG(x) (cos((x) * M_PI / 180.0))
#define SIN_DEG(x) (sin((x) * M_PI / 180.0))
#define TAN_DEG(x) (tan((x) * M_PI / 180.0))

#define MAX_INT std::numeric_limits<int>::max()
#define MAX_WERT(a, b) ( (a) > (b) ? (a) : (b) )
#define MIN_WERT(a, b) ( (a) < (b) ? (a) : (b) )

struct schemPoint
{
public:
	schemPoint(){reset();};
	schemPoint(double xcoord, double ycoord){reset(); x = xcoord; y = ycoord;}
private:
	void reset(){x = 0; y = 0; index = -1; hKante = false; opt_winkel = -1;};

public:
	int index;
	double x, y;

	bool hKante;
	double opt_winkel;
	int in_cat;
};

schemPoint rotate_vec(schemPoint vec, double winkel)
{
	schemPoint result = vec;
	winkel = -winkel;
	result.x = COS_DEG(winkel) * vec.x - SIN_DEG(winkel) * vec.y;
	result.y = SIN_DEG(winkel) * vec.x + COS_DEG(winkel) * vec.y;

	return result;
}

const bool smaller_y2(const schemPoint& p1, const schemPoint& p2)
{
	return (p2.y < p1.y);
}


struct two_dim_vec
{
	two_dim_vec(double xCoord, double yCoord){x = xCoord; y = yCoord; error = false;}
	two_dim_vec(){error = false;}
	double x, y;
	bool error;
};

double winkel_vec2(two_dim_vec a)
{
	if (a.x == 0 && a.y > 0)
		return 90.0;
	else if (a.x == 0 && a.y < 0)
		return 270.0;
	else if (a.x == 0 && a.y == 0)
	{
		cout << "nv" << endl;
		return 0.0; //hack..
	}

	double result = atan(a.y / a.x);
	result *= 180.0/M_PI;

	if (a.x > 0 && a.y < 0)
		result = 270 + (90 + result);
	else if (a.x < 0 && a.y > 0)
			result = 90 + result + 90;
	else if (a.x < 0 && a.y < 0)
		result = 180.0 + result;

	return result;
}



enum EDGE_TYPE {SV_EDGE = 1000, V_EDGE, H_EDGE, SH_EDGE};

struct pathEdge
{
	int left_index;
	int right_index;

	int upper_index;
	int lower_index;

	int edge_type;
	int cost;
};


class dynProgPfad
{
public:
	dynProgPfad()
	{
	}
	

	void doSchematicPath(vector<schemPoint>& pfad, vector<schemPoint>& pfad_schematic, vector<double>& valid_angles, double min_x, double min_y, double shrink_factor)
	{
		//Test auf Monotonie
		bool x_monotone[2] = {true, true};
		bool y_monotone[2] = {true, true};

		for (unsigned i = 0; i < pfad.size() - 1; i++)
		{
			if (pfad[i].x > pfad[i + 1].x)
				x_monotone[0] = false;

			if (pfad[i].x < pfad[i + 1].x)
				x_monotone[1] = false;

			if (pfad[i].y > pfad[i + 1].y)
				y_monotone[0] = false;

			if (pfad[i].y < pfad[i + 1].y)
				y_monotone[1] = false;
		}
		if (!x_monotone[0] && !y_monotone[0] && !x_monotone[1] && !y_monotone[1])
		{
			cout << "SEVERE ERROR: Kein monotoner Pfad!" << endl;
			return;
		}

		unsigned int n = pfad.size();

				/* Koordinaten anpassen */

		for (size_t i = 0; i < n; i++) {
			pfad[i].x = (pfad[i].x - min_x) * shrink_factor;
			pfad[i].y = (pfad[i].y - min_y) * shrink_factor;
		}

		/* Drehe Pfad so dass x-Monotonie von links nach rechts vorliegt */
		double rotate_angle = 0.0;
		if (x_monotone[0])
			rotate_angle = 0.0;
		else if (y_monotone[0])
			rotate_angle = 90.0;
		else if (x_monotone[1])
			rotate_angle = 180;
		else if (y_monotone[1])
			rotate_angle = 270;

		for (unsigned i = 0; i < n; i++)
		{
			pfad[i] = rotate_vec(pfad[i], rotate_angle);
		}

					
		/* Erzeuge Kopie des original pfads. Diese wird das Resultat enthalten */
		pfad_schematic.resize(n);
		copy(pfad.begin(), pfad.end(), pfad_schematic.begin());

		/* Sortiere nach y-Koordinate */
		vector<schemPoint> pfad_s(pfad);
		for (unsigned int i = 0; i < n; i++)
		{
			pfad_s[i].index = i;
		}

		sort(pfad_s.begin(), pfad_s.end(), smaller_y2);

		/* Intervall-Berechnung */
		vector<int> intervalle;
		intervalle.resize(n - 1);

		calcIntervals4(pfad_schematic, pfad_s, intervalle, valid_angles);

		/* Berechne Intervallhöhen */
		vector<double> height_of_intervall(n - 1, 0);
	
		/* Tatsächliche Höhen bestimmen */
		calculateTrueHeights(pfad, pfad_s, intervalle, height_of_intervall, valid_angles);

		/* Tatsächliche Koordinaten bestimmen */
		calculateCoordinates(pfad, pfad_s, height_of_intervall, pfad_schematic, valid_angles);

		/* Zurück rotieren der Koordinaten auf urpsrüngliche Ausrichtung*/
		for (unsigned i = 0; i < n; i++)
		{
			pfad_schematic[i] = rotate_vec(pfad_schematic[i], -rotate_angle);
			pfad[i] = rotate_vec(pfad[i], -rotate_angle);
		}

	}

private:
	void calculateCoordinates(vector<schemPoint>& pfad, vector<schemPoint>& pfad_s, vector<double>& height_of_intervall, vector<schemPoint>& pfad_schematic, vector<double> valid_angles)
	{
		int n = (int)pfad.size();

		/* y-koordinaten festlegen */
		pfad_schematic[pfad_s[n - 1].index].y = 0;
		for (int i = n - 2; i >= 0; i--)
		{
			int index = pfad_s[i].index;
			int index_lower = pfad_s[i + 1].index;
			pfad_schematic[index].y = pfad_schematic[index_lower].y + height_of_intervall[i];
		}

		/* x-koordinaten festlegen */
		pfad_schematic[0].x = 0;
		for (int i = 1; i < n; i++)
		{
			double assigned_angle = pfad_schematic[i - 1].opt_winkel;
			double y_delta = abs(pfad_schematic[i].y - pfad_schematic[i - 1].y);
			double x_delta = 0;

			if (assigned_angle >= 270)
				assigned_angle = 360 - assigned_angle;

			/* h-Kante wird aufgedrückt */
			if (pfad_schematic[i - 1].hKante && y_delta > 0 && assigned_angle == 0)
			{
				if (pfad_schematic[i].y - pfad_schematic[i - 1].y > 0)
					assigned_angle = valid_angles[1];
				else
					assigned_angle = 360 - valid_angles[valid_angles.size() - 2];

			}

			if (y_delta == 0)
				assigned_angle = 0;

			if (assigned_angle == 90.0)
			{
				x_delta = 0;
			}
			else if (assigned_angle == 0.0)
			{

				x_delta = schematization::category_to_min_length[pfad[i].in_cat];
			}
			else
			{
				x_delta = y_delta * 1.0/TAN_DEG(assigned_angle);
			}

			pfad_schematic[i].x = x_delta + pfad_schematic[i - 1].x;
		}

		/*
		cout << "PFAD: " << endl;
		for (int i = 0; i < n; i++)
		{
			cout << pfad_schematic[i].x << " " << pfad_schematic[i].y << endl;
		}
		cout << endl;
		*/
	}
	
	void calculateTrueHeights(vector<schemPoint>& pfad, vector <schemPoint> pfad_s, vector<int>& intervalle, vector<double>& height_of_intervall, vector<double>& valid_angles)
	{
		/* index auf sortierte Knoten*/
		for (unsigned int i = 0; i < pfad.size(); i++)
			pfad[pfad_s[i].index].index = i;

		int nmbOneIntervalls = 0;
		int n = pfad.size();
		vector<int> translator(n - 1, -1);

		for (int i = 0; i < n - 1; i++)
		{
			if (intervalle[i] != 0)
			{
				nmbOneIntervalls++;
				translator[i] = nmbOneIntervalls;
			}

			double winkel = chooseBestAngle(pfad[i], pfad[i + 1], valid_angles);
		    if (i > 0){
		        double winkel_last = chooseBestAngle(pfad[i - 1], pfad[i], valid_angles);
		        //cout << "winkel: " << winkel_last << ", " << winkel << endl;
		        if (winkel_last == 90.0 && winkel == 270.0){
		            double angle_larger_270 = 359.99;
                    for (int j = 0; j < valid_angles.size(); j++){
                        if (valid_angles[j] > 270.0 && valid_angles[j] < angle_larger_270)
                            angle_larger_270 = valid_angles[j];
                    }
                    winkel = angle_larger_270;
		        }
		        else if (winkel_last == 270.0 && winkel == 90.0){
		            double angle_smaller_90 = 0;
                    for (int j = 0; j < valid_angles.size(); j++){
                        if (valid_angles[j] < 90.0 && valid_angles[j] > angle_smaller_90)
                            angle_smaller_90 = valid_angles[j];
                    }
                    winkel = angle_smaller_90;
		        }
		        //cout << winkel_last << ", " << winkel << endl;
                if (chooseBestAngle(pfad[i], pfad[i + 1], valid_angles) != winkel){
                    cout << "conflicting angles! chose new angle: " << winkel << endl;
                }

		    }

			pfad[i].hKante = (winkel == 0.0 || winkel == 180.0);
			pfad[i].opt_winkel = winkel;

			int start_index = pfad[i].index;
			int end_index 	= pfad[i + 1].index;

			if (pfad[i].hKante && pfad[i].opt_winkel == 0.0)
			{
				for (int j = MIN_WERT(start_index, end_index); j < MAX_WERT(start_index, end_index); j++)
				{
					if (intervalle[j] != 0)
					{
						/* Kleinsten Winkel annehmen */
						if (start_index > end_index)
							pfad[i].opt_winkel = valid_angles[1];
						else
							pfad[i].opt_winkel = valid_angles[valid_angles.size() - 2];
					}
				}

			}
			else if  (!pfad[i].hKante)
			{
				bool all_zero = true;
				for (int j = MIN_WERT(start_index, end_index); j < MAX_WERT(start_index, end_index); j++)
				{
					if (intervalle[j] != 0)
						all_zero = false;
				}

				if (all_zero)
					pfad[i].opt_winkel = 0.0;
			}
		}


		//LP - Gurobi
		GRBEnv gurobi_env;
		gurobi_env.set(GRB_IntParam_OutputFlag, 0);
		GRBModel gurobi_model(gurobi_env);
		
		//Variables:
		vector <GRBVar> strip_heights;		
		strip_heights.resize(pfad.size() - 1);
		vector <double> obj_coef(pfad.size() - 1, 0);
		
		for (int i = 0; i < (int)pfad.size() - 1; i++)
		{
			double winkel = pfad[i].opt_winkel;
			//cout << "grb_new winkel: " << winkel << endl;
			
			if (winkel >= 270)
				winkel = 360 - winkel;

			int start_index = pfad[i].index;
			int end_index = pfad[i + 1].index;

			for (int j = MIN_WERT(start_index, end_index); j < MAX_WERT(start_index, end_index); j++)
			{
				if (intervalle[j] != 0)
				{
					obj_coef[j] += 1.0/(abs(SIN_DEG(winkel)));
				}
			}
		}

		
		for (size_t i = 0; i < pfad.size() - 1; ++i) {
			if (intervalle[i] == 0)
				strip_heights[i] = gurobi_model.addVar(0.0, 0.0, 0.0, GRB_CONTINUOUS, "strip_heights[" + boost::lexical_cast<string>(i) + "]-0");
			else
			{
				strip_heights[i] = gurobi_model.addVar(0.0, MAX_INT, obj_coef[i], GRB_CONTINUOUS, "strip_heights[" + boost::lexical_cast<string>(i) + "]");
			}
		}
		gurobi_model.update();

		/* Constraints */
		for (unsigned int i = 0; i < pfad.size() - 1; i++)
		{
			/* Merke die Anzahl der 1 Intervalle pro Kante */
			int index = pfad[i].index;
			int r_index = pfad[i + 1].index;

			int counter = 0;
			GRBLinExpr height = 0;
			for (int j = MIN_WERT(index, r_index); j < MAX_WERT(index, r_index); j++)
			{
				if (intervalle[j] != 0)
				{
					counter++;
					height += strip_heights[j];
				}
			}

			
			if (counter > 0) //Wenigstens ein 1-Intervall gefunden
			{
				double winkel = pfad[i].opt_winkel;
				//cout << "winkel grb: " << winkel << endl;

				winkel = (winkel >= 270) ? 360 -  winkel : winkel;

				double mindest_abst = schematization::category_to_min_length[pfad[i+1].in_cat]  * abs(SIN_DEG(winkel));
				
				if (mindest_abst <= 1e-12) //sometimes the result of say sin(0) is not exactly 0 but something like 6e-15 
					mindest_abst = 0.0;
				
				
				gurobi_model.addConstr(height >= mindest_abst);
			}
		}	

		for (unsigned int i = 0; i < pfad.size() - 1; i++){
			if (intervalle[i] == 0)
				gurobi_model.addConstr(strip_heights[i] == 0);

		}

		/* Modellbildung abgeschlossen */
		

		//Lösen
		gurobi_model.optimize();

				
		if (gurobi_model.get(GRB_IntAttr_Status) == GRB_OPTIMAL){
			/* Zurückübersetzen der Lösung */
			int current_pos = 0;
			for (unsigned int i = 0; i < height_of_intervall.size(); i++)
			{
				if (intervalle[i] != 0)
				{
					height_of_intervall[i] = strip_heights[i].get(GRB_DoubleAttr_X);
					assert(height_of_intervall[i] >= 0.0);
					current_pos++;				
				}
				gurobi_model.write("ausgabe.lp");
				gurobi_model.write("ausgabe.sol");
			}
		}
		else{
			cout << "gurobi failed" << endl;
		}
	}

	void calcIntervals4(vector<schemPoint>& pfad, vector<schemPoint>& pfad_s, vector<int>& intervalle, vector<double>& valid_angles)
	{
		int n = pfad.size();
		for (int i = 0; i < n - 1; i++)
		{
			double winkel = chooseBestAngle(pfad[i], pfad[i + 1], valid_angles);
		    if (i > 0){
		        double winkel_last = chooseBestAngle(pfad[i - 1], pfad[i], valid_angles);
		        if (winkel_last == 90.0 && winkel == 270.0){
		            double angle_larger_270 = 359.99;
                    for (int j = 0; j < valid_angles.size(); j++){
                        if (valid_angles[j] > 270.0 && valid_angles[j] < angle_larger_270)
                            angle_larger_270 = valid_angles[j];
                    }
                    winkel = angle_larger_270;
		        }
		        else if (winkel_last == 270.0 && winkel == 90.0){
		            double angle_smaller_90 = 0;
                    for (int j = 0; j < valid_angles.size(); j++){
                        if (valid_angles[j] < 90.0 && valid_angles[j] > angle_smaller_90)
                            angle_smaller_90 = valid_angles[j];
                    }
                    winkel = angle_smaller_90;
		        }
                }


			pfad[i].hKante = (winkel == 0.0 || winkel == 180.0);
			pfad[i].opt_winkel = winkel;
		}

		pfad[n - 1].opt_winkel = -1.0;
		pfad[n - 1].hKante = false;

		intervalle.resize(n - 1);
		for (int i = 0; i < n - 1; i++)
		{
			intervalle[i] = 0;
		}

		/* Bereite mincost, etc. vor */
		m_mincost.resize(n);
		m_minindex.resize(n);
		m_vcount.resize(n);

		for (int i = 0; i < n; i++)
		{
			pfad[pfad_s[i].index].index = i;
			m_mincost[i] = -1;
			m_minindex[i] = -1;
			m_vcount[i] = -1;
		}

		/* Kanten anlegen */
		int* edges_srt_upper = new int[n * 2];
		int* edges_srt_lower = new int[n * 2];

		for (int i = 0; i < n * 2; i++)
		{
			edges_srt_upper[i] = -1;
			edges_srt_lower[i] = -1;
		}

		//Erstelle Container für Kanten
		edges.resize(n - 1);

		for (int i = 0; i < n - 1; i++)
		{
			pathEdge e;
			e.left_index = i;
			e.right_index = i + 1;

			e.upper_index = MAX_WERT(pfad[i].index, pfad[i + 1].index);
			e.lower_index = MIN_WERT(pfad[i].index, pfad[i + 1].index);

			if (pfad[i].x == pfad[i + 1].x)
			{
				e.edge_type = SV_EDGE;
				e.cost = MAX_INT;
			}
			else if (pfad[i].y == pfad[i + 1].y)
			{
				e.edge_type = SH_EDGE;
				e.cost = MAX_INT;
			}
			else
			{
				double winkel = chooseBestAngle(pfad[i], pfad[i+1], valid_angles);
				e.edge_type = (winkel == 0.0 || winkel == 180.0) ? H_EDGE : V_EDGE;

				if (e.edge_type == H_EDGE)
				{
					e.cost = 1;
				}
				else if (e.edge_type == V_EDGE)
				    e.cost = 1;
				else
					assert(false);
			}

			edges[i] = e;

			if (edges_srt_upper[e.upper_index * 2] == -1)
			{
				edges_srt_upper[e.upper_index * 2] = i;
			}
			else
			{
				edges_srt_upper[e.upper_index * 2 + 1] = i;
			}

			if (edges_srt_lower[e.lower_index * 2] == -1)
			{
				edges_srt_lower[e.lower_index * 2] = i;
			}
			else
			{
				edges_srt_lower[e.lower_index * 2 + 1] = i;
			}
		}

		//m_vcount bestimmen. m_vcount entspricht der kosten für das setzen auf 0 von n - 1 bis j;
		int vcount = 0;

		int svcounter = 0;

		for (int i = n - 1; i >= 0; i--)
		{
			int index[2] = {edges_srt_lower[i * 2], edges_srt_lower[i * 2 + 1]};

			int cost = 0;
			for (int j = 0; j < 2; j++)
			{
				if (index[j] == -1)
					break;

				if (edges[index[j]].edge_type == SV_EDGE)
				{
					svcounter++;
				}
				else if (edges[index[j]].edge_type == V_EDGE)
				{
					cost += edges[index[j]].cost;
				}

			}
			vcount += cost;
			assert(vcount >= 0);

			if (svcounter == 0)
			{
				m_vcount[i] = vcount;
			}
			else
			{
				m_vcount[i] = MAX_INT;
			}
		}

		int cost = 0;
		cost = computeCost(edges_srt_upper, edges_srt_lower, 0, n - 1);

		//Rekonstruktion der Intervalle

		if (m_minindex[0] != -1)
		{
			int index = m_minindex[0];
			do
			{
				intervalle[index] = 1;
				index = m_minindex[index + 1];
			}
			while (index != -1);
		}

		delete[] edges_srt_upper;
		delete[] edges_srt_lower;
	}

	int computeCost(int* edges_srt_upper, int* edges_srt_lower, int ll, int ul)
	{
		if (ul == ll)
		{
			m_mincost[ul] = 0;
			return 0;
		}
		int vcost = 0;
		int hcost = 0;

		int sv_counter = 0;
		int sh_counter = 0;

		int min_c = m_vcount[ll];
		int min_index = -1;

		/*
		 * Kosten:
		 * - (s)h-Kanten, die über "i" gehen
		 * - (s)v-Kanten die zw. ul und i liegen
		 * - Rest von i bis ll
		 * */

		for (int i = ll; i < ul; i++)
		{
			//Bestimmung der vcount und hcount Werte
			int edges_end  [2] = {edges_srt_upper[i * 2], edges_srt_upper[i * 2 + 1]};
			int edges_start[2] = {edges_srt_lower[i * 2], edges_srt_lower[i * 2 + 1]};

			for (int j = 0; j < 2; j++)
			{
				/* Kanten die auf 0 gesetzt werden */
				if (edges_end[j] != -1)
				{
					pathEdge& e = edges[edges_end[j]];

					if (e.lower_index >= ll)
					{
						if (e.edge_type == V_EDGE)
						{
							vcost += e.cost;
						}
						else if (e.edge_type == SV_EDGE)
							sv_counter++;
						else if (e.edge_type == SH_EDGE)
							sh_counter--;
						else if (e.edge_type == H_EDGE)
						{
							assert(hcost > 0);
							hcost -= e.cost;
						}
					}
				}

				/* Kanten die auf 1 gesetzt werden */
				if (edges_start[j] != -1)
				{
					pathEdge& e = edges[edges_start[j]];

					if (e.edge_type == SV_EDGE)
						sv_counter--;
					else if (e.edge_type == SH_EDGE)
						sh_counter++;
					else if (e.edge_type == H_EDGE)
						hcost += e.cost;
				}

			}

			//Bestimmung der totalen Kosten für i
			if (m_mincost[i + 1] == -1)
				computeCost(edges_srt_upper, edges_srt_lower, i+1, ul);

			assert(m_mincost[i+1] >= 0);

			int c = 0;
			if (sv_counter > 0 || sh_counter > 0)
				c = MAX_INT;
			else
			{
				c = vcost + hcost + m_mincost[i + 1];
				assert(vcost >= 0);
				assert(hcost >= 0);
				assert(m_mincost[i + 1]>= 0);
			}

			assert(c != -1);

			if (min_c > c)
			{
				min_c = c;
				min_index = i;
			}
		}

		assert(min_c != -1);

		m_mincost[ll] = min_c;
		m_minindex[ll] = min_index;

		return min_c;
	}

	inline double chooseBestAngle(schemPoint prev, schemPoint current, vector<double>& valid_angles)
	{
		two_dim_vec rv(current.x - prev.x, current.y - prev.y);
		if (rv.x == 0.0 && rv.y == 0.0)
		{
			//			cout << "0.0" << endl;
			return 0.0;
		}
		double winkel = winkel_vec2(rv);
		double best_angle = -1.0;

		if (winkel < valid_angles[0])
			return valid_angles[0];

		for (unsigned int i = 1; i < valid_angles.size(); i++)
		{
			if (winkel >= valid_angles[i - 1] && winkel < valid_angles[i])
			{
				best_angle = (winkel - valid_angles[i - 1] < valid_angles[i] - winkel) ? valid_angles[i - 1]: valid_angles[i];

				if (best_angle == 360.0)
					best_angle = 0.0;

				break;
			}
		}

		if (best_angle == -1)
		{
			cout << current.x << " " << current.y << "\t " << prev.x << " " << prev.y << endl;
			cout << "WINKEL: " << winkel << endl;
			assert(best_angle != -1);
		}
		return best_angle;
	}

private:
	vector < pathEdge > edges;
	vector < int > m_mincost;
	vector < int > m_minindex;
	vector < int > m_vcount;

	
};

#endif /* PFADDYNPROG_H_ */
