/******************************************************************************
* schemePathWrapper.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
#ifndef SCHEMEPATHWRAPPER_H_
#define SCHEMEPATHWRAPPER_H_

#include <boost/foreach.hpp>
#include <boost/multi_array.hpp>
#include <boost/graph/graph_traits.hpp>
#include <boost/graph/iteration_macros.hpp>

#include "pfadDynProg.h"

#include <queue>
#include <map>
#include <utility>
#include <set>

#include "../geometry/rectangle.h"
#include "../algorithms/draw.h"

enum MONOTONICITY {LEFT_TO_RIGHT = 123, RIGHT_TO_LEFT, TOP_TO_BOTTOM, BOTTOM_TO_TOP};

#define MAX(a, b) ( (a) < (b) ) ?  (b) : (a) 

struct Axismonotone_stats{
  int nmb_paths;
  int nmb_edges;
  int nmb_out_edges;
  int nmb_added_edges;
  int nmb_pairs_oo_preserved;
  int nmb_pairs_oo_violated;

};

class dummyPathBox
{
 public:
  dummyPathBox(bool three_edges, MONOTONICITY in, MONOTONICITY out){
    is_three_edges = three_edges;
    points.push_back(schematization::point(0, 0));
    points.push_back(schematization::point(0, 0));
    points.push_back(schematization::point(0, 0));

    if (three_edges)
      points.push_back(schematization::point(0, 0));
           
    m_in = in;
    m_out = out;
        
    x = 0;
    y = 0;
  }

  float getLastVertexX(){
    return points[points.size() - 1].x;
  }
  float getLastVertexY(){
    return points[points.size() - 1].y;
  }

  void increaseWidth(float delta_w)
  {
    if (m_in == LEFT_TO_RIGHT || m_in == RIGHT_TO_LEFT){
                
      points[1].x += delta_w;
      points[2].x += delta_w;
    }
    else if(m_in == TOP_TO_BOTTOM || m_in == BOTTOM_TO_TOP){

      points[2].x += delta_w;
            
      if (m_out == TOP_TO_BOTTOM || m_out == BOTTOM_TO_TOP)
	points[3].x += delta_w;        
    }
  }
    
  void increaseHeight(float delta_h)
  {
    if (m_in == LEFT_TO_RIGHT || m_in == RIGHT_TO_LEFT){
                
      points[2].y += delta_h;
            
      if (m_out == LEFT_TO_RIGHT || m_out == RIGHT_TO_LEFT)
	points[3].y += delta_h;
    }
    else if(m_in == TOP_TO_BOTTOM || m_in == BOTTOM_TO_TOP){
                
      points[1].y += delta_h;
      points[2].y += delta_h;
    }
  }

  void getDistinctPoints(vector<schematization::point>& p){
    //get distinct points except first one
    for (int i = 1; i < points.size(); i++){
      if (points[i].x != points[i-1].x || points[i].y != points[i-1].y){
	p.push_back(points[i]);
      }
    }
        
  }
    
  float getX(){
    return x;
  }
   
  float getY(){
    return y;
  }
    
  void setXY(float x, float y) {
    this-> x = x;
    this-> y = y;
  } 
   
  void moveBy(float x, float y){
    this->x += x;
    this->y += y;
  }


  bool is_three_edges;
    
  MONOTONICITY m_in;
  MONOTONICITY m_out;

  float x, y;
  vector <schematization::point> points;
};

class pathBox
{
 public:
  pathBox(){
    x = 0; y = 0;
    dPathBox = NULL;determineSegmentsCalled = false;
  }

  bool intersectsPathBox(pathBox& p)
  {
	    
    double AX1 = x + min_x;
    double AX2 = x + max_x;
    double AY1 = y + min_y;
    double AY2 = y + max_y;
    
    double BX1 = p.x + p.min_x;
    double BX2 = p.x + p.max_x;
    double BY1 = p.y + p.min_y;
    double BY2 = p.y + p.max_y;
        

    return (AX1 < BX2) && (AX2 > BX1) && 
      (AY1 < BY2) && (AY2 > BY1);
                            
  }
	
	
  bool intersectsPriorPathBox(pathBox& p){
    bool obvious_conflict = intersectsPathBox(p);	
    if (obvious_conflict)
      return true;
       
    if (!determineSegmentsCalled)
      determineSegments();
       
    if (!p.determineSegmentsCalled)
      p.determineSegments();
                
            
    for (int i = 0; i < segments.size(); i++){
      for (int j = 0; j < p.segments.size(); j++){
	double AX1 = segments[i].first.x + x;     
	double AX2 = segments[i].second.x + x;     
	double AY1 = segments[i].first.y + y;     
	double AY2 = segments[i].second.y + y;     

	double BX1 = p.segments[j].first.x + p.x;     
	double BX2 = p.segments[j].second.x + p.x;     
	double BY1 = p.segments[j].first.y + p.y;     
	double BY2 = p.segments[j].second.y + p.y;     
            
	if ( (AX1 <= BX2) && (AX2 >= BX1) && 
	     (AY1 <= BY2) && (AY2 >= BY1) )
	  return true;
       
      }
    }
    return false;
            
      
  }
	
	
  float getLastVertexX(){
    return points[points.size() - 1].x;
  }
  float getLastVertexY(){
    return points[points.size() - 1].y;
  }
    
  float getFirstVertexX(){
    return points[0].x;
  }
  float getFirstVertexY(){
    return points[0].y;
  }
    
  schematization::point getLL(void){
    return schematization::point(x + min_x, y + min_y);
  }
  schematization::point getUR(void){
    return schematization::point(x + max_x, y + max_y);
  }


  void addPoint(float x, float y)
  {
	
    if (points.size() == 0)
      {
	delta_x = x; 
	delta_y = y;
	x = 0; 
	y = 0;
	max_x = 0; 
	min_x = 0;
			
	max_y = 0; 
	min_y = 0;
      }
    else
      {
	x -= delta_x; 
	y -= delta_y;
	max_x = (max_x > x) ? max_x : x;
	min_x = (min_x < x) ? min_x : x;
	max_y = (max_y > y) ? max_y : y;
	min_y = (min_y < y) ? min_y : y;

      }
    points.push_back(schematization::point(x, y));
  }
    
  void moveBy(float x, float y)
  {
    this->x += x;
    this->y += y;
    if (dPathBox!= NULL)
      dPathBox->moveBy(x, y);
  }
   
  void setXY(float x, float y) {
    this-> x = x;
    this-> y = y;
    if (dPathBox!= NULL)
      dPathBox->setXY(x, y);
  } 
    
  int getX(){return x;};
  int getY(){return y;};


  float getWidth(){return max_x - min_x;}
  float getHeight(){return max_y - min_y;}
  
  void determineSegments(){
    determineSegmentsCalled = true;
                
    if (points[0].x == min_x && points[1].x == min_x){
      schematization::point p0((points[1].x - points[0].x)/100.0 + points[0].x, points[0].y);  
      segments.push_back(make_pair(p0, points[1]));
    }
    if (points[0].y == min_y && points[1].x == min_y){
      schematization::point p0(points[0].x, (points[1].y - points[0].y)/100.0 + points[0].y);  
      segments.push_back(make_pair(p0, points[1]));
    }        
    if (points[0].x == max_x && points[1].x == max_x){
      schematization::point p0((points[1].x - points[0].x)/100.0 + points[0].x, points[0].y);  
      segments.push_back(make_pair(p0, points[1]));
    }
    if (points[0].y == max_y && points[1].x == max_y){
      schematization::point p0(points[0].x, (points[1].y - points[0].y)/100.0 + points[0].y);  
      segments.push_back(make_pair(p0, points[1]));
            
    }        
        
    int i = 1;
    while (i < points.size() - 2){
      bool increment = false;
      if (points[i].x == min_x){
	if (points[i + 1].x == min_x){
	  segments.push_back(make_pair(points[i], points[i + 1]));
	  increment = true;
	}
	else
	  segments.push_back(make_pair(points[i], points[i]));
                
      }
      if (points[i].x == max_x){
	if (points[i + 1].x == max_x){
	  segments.push_back(make_pair(points[i], points[i + 1]));
	  increment = true;
	}
	else
	  segments.push_back(make_pair(points[i], points[i]));
                
      }
      if (points[i].y == min_y){
	if (points[i + 1].y == min_y){
	  segments.push_back(make_pair(points[i], points[i + 1]));
	  increment = true;
	}
	else
	  segments.push_back(make_pair(points[i], points[i]));
                
      }

      if (points[i].y == max_y){
	if (points[i + 1].y == max_y){
	  segments.push_back(make_pair(points[i], points[i + 1]));
	  increment = true;
	}
	else
	  segments.push_back(make_pair(points[i], points[i]));
                
      }
            
      i++;
      if (increment)
	i++;
       
    }
        
    int last_node = points.size() - 1;
    int second_last_node = last_node - 1;
        
    if (points[last_node].x == min_x && points[second_last_node].x == min_x){
      schematization::point p0((points[second_last_node].x - points[last_node].x)/100.0 + points[last_node].x, points[last_node].y);  
      segments.push_back(make_pair(points[second_last_node], p0));
    }
    if (points[last_node].y == min_y && points[second_last_node].x == min_y){
      schematization::point p0(points[last_node].x, (points[second_last_node].y - points[last_node].y)/100.0 + points[last_node].y);  
      segments.push_back(make_pair(points[second_last_node], p0));
    }        
    if (points[last_node].x == max_x && points[second_last_node].x == max_x){
      schematization::point p0((points[second_last_node].x - points[last_node].x)/100.0 + points[last_node].x, points[last_node].y);  
      segments.push_back(make_pair(points[second_last_node], p0));
    }
    if (points[last_node].y == max_y && points[second_last_node].x == max_y){
      schematization::point p0(points[last_node].x, (points[second_last_node].y - points[last_node].y)/100.0 + points[last_node].y);  
      segments.push_back(make_pair(points[second_last_node], p0));
    }            
        
    
  }


 public:
  MONOTONICITY monotonicity;

  dummyPathBox* dPathBox;
    
  //private:
  int x, y;
  vector <schematization::point> points;

  float max_x;
  float max_y;
  float min_x;
  float min_y;
  float delta_x, delta_y;
	
  typedef std::pair<schematization::point, schematization::point> segment;
  vector <segment> segments;
	
  bool determineSegmentsCalled;
	
};



template<typename graphType>
class schemePathWrapper{
 public:

  typedef typename graphType::vertex_descriptor vertex_descriptor;
  typedef typename graphType::edge_descriptor edge_descriptor;
  typedef typename graph_traits<graphType>::out_edge_iterator out_edge_iterator;
  typedef typename graph_traits<graphType>::in_edge_iterator in_edge_iterator;

 public:
  schemePathWrapper(){
		
		
  }

  void schematize(graphType& in_g, graphType& out_g, double angle, bool simple_join = true)
  {	
		
    /* Koordinaten anpassen */
    double min_y = in_g[(vertex_descriptor)0].coord.y;
    double min_x = in_g[(vertex_descriptor)0].coord.x;
    double max_y(min_y), max_x(min_x);
		
    BGL_FORALL_VERTICES_T(v, in_g, graphType) {
      min_y = (min_y > in_g[v].coord.y ? in_g[v].coord.y : min_y);
      min_x = (min_x > in_g[v].coord.x ? in_g[v].coord.x : min_x);

      max_y = (max_y < in_g[v].coord.y ? in_g[v].coord.y : max_y);
      max_x = (max_x < in_g[v].coord.x ? in_g[v].coord.x : max_x);
				
    }
		
    double max_delta = MAX_WERT(max_x - min_x, max_y - min_y);

    double shrink_factor = 1000.0/max_delta;

		
    /*Graph splitten*/
    split_graph(in_g);
		
    /*Gültige Winkel*/
    vector <double> valid_angles(1, 0);
    const double smallest_angle = angle;

    while (angle < 360.0){
      valid_angles.push_back(angle);
      angle += smallest_angle;
    }

    valid_angles.push_back(360.0);
	
    /*Gesplittete Pfade einzeln schematisieren*/
    dynProgPfad dpp;
    paths_schem.resize(paths.size());
    for (size_t i = 0; i < paths.size(); i++){
      dpp.doSchematicPath(paths[i], paths_schem[i], valid_angles, min_x, min_y, shrink_factor);
    }

    /* Koordinaten runden */
    for (size_t i = 0; i < paths_schem.size(); i++){
      for (int j = 0; j<  paths_schem[i].size(); j++){
	paths_schem[i][j].x = round(paths_schem[i][j].x);
	paths_schem[i][j].y = round(paths_schem[i][j].y);
      }
    }
        
		
    /*Zusammenfügen der Pfade*/
    cout << "merge started" << endl;
    out_g = in_g;
    if (simple_join)
      mergePathsSimple(out_g);
    else
      mergePathsIntersectionFree(out_g);
    cout << "merge ended" << endl;
		    
    /* Statistik stuff */
    cout << "stats" << endl;
    nmb_edges = num_edges(in_g);
    nmb_out_edges = num_edges(out_g);

    nmb_pairs_oo_preserved = 0;
    nmb_pairs_oo_violated = 0;

    {
      vertex_descriptor v, u;
      set<vertex_descriptor> visited_vertices;
      BGL_FORALL_VERTICES_T(v, in_g, graphType){
	visited_vertices.insert(v);
	BGL_FORALL_VERTICES_T(u, in_g, graphType){
	  if (visited_vertices.find(u) != visited_vertices.end())
	    continue;     

	  double dx1 = in_g[v].coord.x - in_g[u].coord.x;
	  double dy1 = in_g[v].coord.y - in_g[u].coord.y;
	  double dx2 = out_g[v].coord.x - out_g[u].coord.x;
	  double dy2 = out_g[v].coord.y - out_g[u].coord.y;
        
	  if ( ((dx1 <= 0 && dx2 <= 0) || (dx1 >= 0 && dx2 >= 0)) &&
	       ((dx1 <= 0 && dx2 <= 0) || (dx1 >= 0 && dx2 >= 0)) )
	    nmb_pairs_oo_preserved++;
	  else   
	    nmb_pairs_oo_violated++;
	}
      }
    }
	
  }
		
		
  void getStatistics(Axismonotone_stats& s){
    s.nmb_paths = paths.size() + 1;
    s.nmb_added_edges = this->nmb_added_edges;
    s.nmb_edges = this->nmb_edges;
    s.nmb_out_edges = this->nmb_out_edges;
    s.nmb_pairs_oo_preserved = this->nmb_pairs_oo_preserved;
    s.nmb_pairs_oo_violated = this->nmb_pairs_oo_violated;
	    
  } 

 protected:

  void split_graph(graphType& in_g)
  {
    vertex_descriptor source = find_start_vertex(in_g);

    queue<vertex_descriptor> nodes;
    nodes.push(source);

    int path_counter = 0;

    while(!nodes.empty()){

      vertex_descriptor v_start = nodes.front();
      nodes.pop();

      paths.resize(path_counter + 1);
      vd_paths.resize(path_counter + 1);
      vd_paths_stats.resize(path_counter + 1);

      //Flags: 1 x monotonie lr, 1 << 1 x monotonie rl, 1 << 2 y mon uo, 1 << 3 y mon ou
      int monotonicity = 1 | 1 << 1 | 1 << 2 | 1 << 3;

      vertex_descriptor v_current = v_start;
      vertex_descriptor v_prev = -1;

      vector <vertex_descriptor> out_edges_vec;
      int in_cat = -1;
      edge_descriptor out_edge_desc;

      while(monotonicity != 0) {

	schemPoint s(in_g[v_current].coord.x, in_g[v_current].coord.y);
	s.in_cat = in_cat;
				
	paths[paths.size() - 1].push_back(s);

	vd_paths[paths.size() - 1].push_back(v_current);
	vd_paths_stats[paths.size() - 1].push_back(v_current);
                
	int nmb_in_edges(0), nmb_out_edges(0);
	out_edges_vec.clear();

				
	BGL_FORALL_OUTEDGES_T(v_current, e, in_g, graphType) {
	  nmb_out_edges++;               
	  out_edges_vec.push_back(target(e, in_g));
	  out_edge_desc = e;
	}
				
	BGL_FORALL_INEDGES_T(v_current, e, in_g, graphType) {
	  nmb_in_edges++;				
	}

	assert(nmb_in_edges <= 1);
	assert(nmb_out_edges <= 1);			
				
	if (nmb_out_edges == 0)
	  break;

	vertex_descriptor v_target = out_edges_vec[0];
				
	if (in_g[v_target].coord.x < in_g[v_current].coord.x) {
	  monotonicity &=  (0 | 1 << 1 | 1 << 2 | 1 << 3);
	}
	else if (in_g[v_target].coord.x > in_g[v_current].coord.x) {
	  monotonicity &=  (1 | 0 << 1 | 1 << 2 | 1 << 3);
	}

	if (in_g[v_target].coord.y < in_g[v_current].coord.y) {
	  monotonicity &=  1 | 1 << 1 | 0 << 2 | 1 << 3;
	}
	else if (in_g[v_target].coord.y > in_g[v_current].coord.y) {
	  monotonicity &=  1 | 1 << 1 | 1 << 2 | 0 << 3;
	}
				
	v_prev = v_current;
	v_current = v_target;

	in_cat = in_g[out_edge_desc].category;

      }

			
      if (out_edges_vec.size() == 1) {
      }
      else if (out_edges_vec.size() == 0 && monotonicity != 0) {
	break;
      }
      else if (out_edges_vec.size() == 0 && monotonicity == 0) {
	assert(false);
      }
      else {
	assert(false);
      }
      
      nodes.push(v_prev); 
			
      path_counter++;

    }
  }

  vertex_descriptor find_start_vertex(const graphType& in_g){
    vertex_descriptor source = (vertex_descriptor)-1;

    vertex_descriptor v;
    BGL_FORALL_VERTICES_T(v, in_g, graphType){
      int nmb_in_edges = 0;
		
      edge_descriptor e;
      BGL_FORALL_INEDGES_T(v, e, in_g, graphType) {
	nmb_in_edges++;
      }
			
      if (nmb_in_edges == 0 && out_degree(v, in_g) != 0) 
	source = v;
    }

    assert(source != -1);
		
    return source;
  }

  void mergePathsSimple(graphType& in_g)
  {
    double cur_x(0.0), cur_y(0.0);
		
    for (size_t i = 0; i < paths.size(); i++){
      double delta_x = paths_schem[i][0].x;
      double delta_y = paths_schem[i][0].y;
			
      for (size_t j = 0; j < paths[i].size(); j++) {
	vertex_descriptor v = vd_paths[i][j];

	in_g[v].coord.x = paths_schem[i][j].x - delta_x + cur_x;
	in_g[v].coord.y = paths_schem[i][j].y - delta_y + cur_y;
      }

      cur_x = paths_schem[i][paths[i].size() - 1].x - delta_x + cur_x;
      cur_y = paths_schem[i][paths[i].size() - 1].y - delta_y + cur_y;
    }
  }



  void mergePathsIntersectionFree(graphType& in_g) {
    /* Bounding Boxes festlegen und Koordinatenursprung pro Box festlegen */
		
    vector<pathBox> pathBoxes;

    for (size_t i = 0; i < paths.size(); i++)  
      {
	pathBox current_pathBox;
	for (size_t j = 0; j < paths_schem[i].size(); j++){
	  current_pathBox.addPoint(paths_schem[i][j].x , paths_schem[i][j].y);
	}
			
	MONOTONICITY m = getMonotonicity(i);
	current_pathBox.monotonicity = m;
	        
	pathBoxes.push_back(current_pathBox);
	pathBox& cur_pathBox = pathBoxes[pathBoxes.size() -1 ];			           		

	attachTwoPaths(i, pathBoxes, cur_pathBox);

	if (m == LEFT_TO_RIGHT){
                
	  //right conflict
	  int rc_index = -1;
	  int rc_min_x = cur_pathBox.getUR().x;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                    
	    if (cur_pathBox.x < p.getUR().x && p.getLL().x < cur_pathBox.getUR().x){
	      if (p.getUR().y > cur_pathBox.y && cur_pathBox.y > p.getLL().y){
		if (p.getLL().x < rc_min_x){
		  rc_index = k;
		  rc_min_x = p.getLL().x;
		}
	      }
	    }
	  }
	  if (rc_index != -1){

	    double delta_x = cur_pathBox.getUR().x - pathBoxes[rc_index].getLL().x + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getLL().x >= rc_min_x){
		moveBox[k] = true;
	      }
	    }
                        
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseWidth(delta_x);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseWidth(delta_x);
		  }
		}
		pathBoxes[k].moveBy(delta_x, 0);
	      }
	    }
	  }
                
                
	  /*lower conflict*/
	  //find upper most box below cur_Box.x, cur_box.y
	  int lc_index = -1;
	  int lc_max_y = cur_pathBox.getLL().y;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                    
	    if (p.getLL().x < cur_pathBox.getUR().x && cur_pathBox.getLL().x < p.getUR().x){
	      if (p.getUR().y < cur_pathBox.y && p.getUR().y > cur_pathBox.getLL().y){
		if (p.getUR().y > lc_max_y){
		  lc_index = k;
		  lc_max_y = p.getUR().y;
		}
	      }
	    }
	  }                
	  if (lc_index != -1){
	    double delta_y = pathBoxes[lc_index].getUR().y - cur_pathBox.getLL().y + 50;//cur_pathBox.getUR().x - pathBoxes[rc_index].getLL().x + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getUR().y <= lc_max_y){
		moveBox[k] = true;
	      }
	    }
                        
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseHeight(delta_y);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseHeight(delta_y);
		  }
		}
		pathBoxes[k].moveBy(0, -delta_y);
	      }
	    }                
                
	  }
        
	  /*upper conflict*/
	  //find upper most box below cur_Box.x, cur_box.y
	  int uc_index = -1;
	  int uc_min_y = cur_pathBox.getUR().y;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                    
	    if (p.getLL().x < cur_pathBox.getUR().x && cur_pathBox.getLL().x < p.getUR().x){
	      if (p.getLL().y > cur_pathBox.y && p.getLL().y < cur_pathBox.getUR().y){
		if (p.getLL().y < uc_min_y){
		  uc_index = k;
		  uc_min_y = p.getLL().y;
		}
	      }
	    }
	  }                
	  if (uc_index != -1){
	    double delta_y = cur_pathBox.getUR().y - pathBoxes[uc_index].getLL().y + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getLL().y >= uc_min_y){
		moveBox[k] = true;
	      }
	    }
                        
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1]){
		    pathBoxes[k - 1].dPathBox->increaseHeight(delta_y);
		  }
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseHeight(delta_y);
		  }
		}
		pathBoxes[k].moveBy(0, delta_y);
	      }
	    }                
	  }      
	}else if (m == RIGHT_TO_LEFT){
                
	  //left conflict
	  int lec_index = -1;
	  int lec_max_x = cur_pathBox.getLL().x;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                    
	    if (cur_pathBox.x > p.getUR().x && p.getUR().x > cur_pathBox.getLL().x){
	      if (p.getUR().y > cur_pathBox.y && cur_pathBox.y > p.getLL().y){
		if (p.getUR().x > lec_max_x){
		  lec_index = k;
		  lec_max_x = p.getUR().x;
		}
	      }
	    }
	  }
	  if (lec_index != -1){
	    double delta_x = pathBoxes[lec_index].getUR().x - cur_pathBox.getLL().x + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
                    
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getUR().x <= lec_max_x){
		moveBox[k] = true;
	      }
	    }
                    
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseWidth(delta_x);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseWidth(delta_x);
		  }
		}
		pathBoxes[k].moveBy(-delta_x, 0);
	      }
	    }
	  }
                
                
	  /*lower conflict*/
	  //find upper most box below cur_Box.x, cur_box.y
	  int lc_index = -1;
	  int lc_max_y = cur_pathBox.getLL().y;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                    
	    if (p.getLL().x < cur_pathBox.getUR().x && cur_pathBox.getLL().x < p.getUR().x){
	      if (p.getUR().y < cur_pathBox.y && p.getUR().y > cur_pathBox.getLL().y){
		if (p.getUR().y > lc_max_y){
		  lc_index = k;
		  lc_max_y = p.getUR().y;
		}
	      }
	    }
	  }                
	  if (lc_index != -1){
	    double delta_y = pathBoxes[lc_index].getUR().y - cur_pathBox.getLL().y + 50;//cur_pathBox.getUR().x - pathBoxes[rc_index].getLL().x + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getUR().y <= lc_max_y){
		moveBox[k] = true;
	      }
	    }
                        
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseHeight(delta_y);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseHeight(delta_y);
		  }
		}
		pathBoxes[k].moveBy(0, -delta_y);
	      }
	    }                
                
	  }
        
	  /*lower conflict*/
	  //find upper most box below cur_Box.x, cur_box.y
	  int uc_index = -1;
	  int uc_min_y = cur_pathBox.getUR().y;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                    
	    if (p.getLL().x < cur_pathBox.getUR().x && cur_pathBox.getLL().x < p.getUR().x){
	      if (p.getLL().y > cur_pathBox.y && p.getLL().y < cur_pathBox.getUR().y){
		if (p.getLL().y < uc_min_y){
		  uc_index = k;
		  uc_min_y = p.getLL().y;
		}
	      }
	    }
	  }                
	  if (uc_index != -1){
	    double delta_y = cur_pathBox.getUR().y - pathBoxes[uc_index].getLL().y + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getLL().y >= uc_min_y){
		moveBox[k] = true;
	      }
	    }
                        
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseHeight(delta_y);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseHeight(delta_y);
		  }
		}
		pathBoxes[k].moveBy(0, delta_y);
	      }
	    }                
	  }
	}else if (m == TOP_TO_BOTTOM){               
	  //upper conflict
	  int uc_index = -1;
	  int uc_max_y = cur_pathBox.getLL().y;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                    
	    if (cur_pathBox.getUR().y > p.getUR().y && p.getUR().y > cur_pathBox.getLL().y){
	      if (p.getUR().x > cur_pathBox.x && cur_pathBox.x > p.getLL().x){
		if (p.getUR().y > uc_max_y){
		  uc_index = k;
		  uc_max_y = p.getUR().y;
		}
	      }
	    }
	  }
	  if (uc_index != -1){
	    double delta_y = pathBoxes[uc_index].getUR().y - cur_pathBox.getLL().y + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
                    
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getUR().y <= uc_max_y){
		moveBox[k] = true;
	      }
	    }
                    
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseHeight(delta_y);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseHeight(delta_y);
		  }
		}
		pathBoxes[k].moveBy(-delta_y, 0);
	      }
	    }
	  }
                
	  /*left conflict*/
	  int lec_index = -1;
	  int lec_max_x = cur_pathBox.getLL().x;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                    
	    if (cur_pathBox.x > p.getUR().x && p.getUR().x > cur_pathBox.getLL().x) {
	      if (p.getUR().y < cur_pathBox.y && p.getUR().y > cur_pathBox.getLL().y){
		if (p.getUR().x > lec_max_x){
		  lec_index = k;
		  lec_max_x = p.getUR().x;
		}
	      }
	    }
	  }                
	  if (lec_index != -1){
	    double delta_x = pathBoxes[lec_index].getUR().x - cur_pathBox.getLL().x + 50;//cur_pathBox.getUR().x - pathBoxes[rc_index].getLL().x + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getUR().x <= lec_max_x){
		moveBox[k] = true;
	      }
	    }
                        
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseWidth(delta_x);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseWidth(delta_x);
		  }
		}
		pathBoxes[k].moveBy(-delta_x, 0);
	      }
	    }                
                
	  }
                
	  /*right conflict*/
	  int rc_index = -1;
	  int rc_min_x = cur_pathBox.getUR().x;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                    
	    if (p.getLL().x > cur_pathBox.x && cur_pathBox.getUR().x > p.getLL().x){
	      if (p.getLL().y > cur_pathBox.y && p.getLL().y < cur_pathBox.getUR().y){
		if (p.getLL().x < rc_min_x){
		  rc_index = k;
		  rc_min_x = p.getLL().x;
		}
	      }
	    }
	  }                
	  if (rc_index != -1){
	    double delta_x = cur_pathBox.getUR().x - pathBoxes[rc_index].getLL().x + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getLL().x <= rc_min_x){
		moveBox[k] = true;
	      }
	    }
                        
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseWidth(delta_x);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseWidth(delta_x);
		  }
		}
		pathBoxes[k].moveBy(delta_x, 0);
	      }
	    }                
                
	  } 
        } else if (m == BOTTOM_TO_TOP){
	  //lower conflict
	  int lc_index = -1;
	  int lc_max_y = cur_pathBox.getLL().y;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                
	    if (p.getLL().x < current_pathBox.x && current_pathBox.x < p.getUR().x){
	      if (p.getUR().y > current_pathBox.getLL().y ){
		if (p.getUR().y > lc_max_y){
		  lc_index = k;
		  lc_max_y = p.getUR().y;
		}
	      }
	    }
	  }
	  if (lc_index != -1){
	    double delta_y = pathBoxes[lc_index].getUR().y - cur_pathBox.getLL().y + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
                
	    for (int k = 0; k < pathBoxes.size() - 1; k++){
	      if (pathBoxes[k].getUR().y >= lc_max_y){
		moveBox[k] = true;
	      }
	    }
                
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseHeight(delta_y);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseHeight(delta_y);
		  }
		}
		pathBoxes[k].moveBy(-delta_y, 0);
	      }
	    }
	  }
            
            
	  /*left conflict*/
	  int lec_index = -1;
	  int lec_max_x = cur_pathBox.getLL().x;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                
	    if (cur_pathBox.x > p.getUR().x && p.getUR().x > cur_pathBox.getLL().x) {
	      if (p.getUR().y < cur_pathBox.y && p.getUR().y > cur_pathBox.getLL().y){
		if (p.getUR().x > lec_max_x){
		  lec_index = k;
		  lec_max_x = p.getUR().x;
		}
	      }
	    }
	  }                
	  if (lec_index != -1){
	    double delta_x = pathBoxes[lec_index].getUR().x - cur_pathBox.getLL().x + 50;//cur_pathBox.getUR().x - pathBoxes[rc_index].getLL().x + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getUR().x <= lec_max_x){
		moveBox[k] = true;
	      }
	    }
                    
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseWidth(delta_x);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseWidth(delta_x);
		  }
		}
		pathBoxes[k].moveBy(-delta_x, 0);
	      }
	    }                
            
	  }
            
	  /*right conflict*/
	  int rc_index = -1;
	  int rc_min_x = cur_pathBox.getUR().x;
	  for (int k = 0; k < pathBoxes.size() - 1; k++){
	    pathBox& p = pathBoxes[k];
                
	    if (p.getLL().x > cur_pathBox.x && cur_pathBox.getUR().x > p.getLL().x){
	      if (p.getLL().y > cur_pathBox.y && p.getLL().y < cur_pathBox.getUR().y){
		if (p.getLL().x < rc_min_x){
		  rc_index = k;
		  rc_min_x = p.getLL().x;
		}
	      }
	    }
	  }                
	  if (rc_index != -1){
	    double delta_x = cur_pathBox.getUR().x - pathBoxes[rc_index].getLL().x + 50;
	    vector<bool> moveBox(pathBoxes.size(), false);
	    for (int k = 0; k < pathBoxes.size() - 2; k++){
	      if (pathBoxes[k].getLL().x <= rc_min_x){
		moveBox[k] = true;
	      }
	    }
                    
	    for (int k = 0; k < moveBox.size(); k++){
	      if (moveBox[k]){
		if (k > 0){
		  if (!moveBox[k - 1])
		    pathBoxes[k - 1].dPathBox->increaseWidth(delta_x);
		}
		if (k < moveBox.size() - 1){
		  if (!moveBox[k + 1]){
		    pathBoxes[k].dPathBox->increaseWidth(delta_x);
		  }
		}
		pathBoxes[k].moveBy(delta_x, 0);
	      }
	    }                
            
                    
	  }          
	} 
      }




    /// Uebertragen der Koordinaten
    for (int i = 0; i < pathBoxes.size(); i++){
      for (size_t j = 0; j < pathBoxes[i].points.size(); j++){
	vertex_descriptor v = vd_paths[i][j];
	in_g[v].coord.x = pathBoxes[i].points[j].x + pathBoxes[i].getX();
	in_g[v].coord.y = pathBoxes[i].points[j].y + pathBoxes[i].getY();
				
      }
			 
			 
      //path link edges  
      if (i != pathBoxes.size() - 1){
	assert(pathBoxes[i].dPathBox != NULL);               

	pathBoxes[i].setXY(  in_g[vd_paths[i][vd_paths[i].size() - 1]].coord.x, 
			     in_g[vd_paths[i][vd_paths[i].size() - 1]].coord.y);

	vector<schematization::point> p;
	pathBoxes[i].dPathBox->getDistinctPoints(p);
        
	if (p.size() == 0)
	  continue;	                            

	vector<vertex_descriptor> new_vertices;
	for (int j = 0; j < p.size(); j++){
	  vertex_descriptor v = add_vertex(in_g);
	  new_vertices.push_back(v);
	  in_g[v].coord.x = pathBoxes[i].dPathBox->getX() + p[j].x;
	  in_g[v].coord.y = pathBoxes[i].dPathBox->getY() + p[j].y;	                
	}
	            
	//first edge
	edge_descriptor e_new;
	bool edge_added;
	tie (e_new, edge_added) = add_edge(vd_paths[i][vd_paths[i].size() - 1], new_vertices[0], in_g);
	in_g[e_new].category = 4;   
	in_g[e_new].non_path_edge = true;
                    
	//add edges between first and last
	for (int j = 0; j < new_vertices.size() - 1; j++){
	  edge_descriptor e;
	  bool edge_added;
	  tie (e, edge_added) = add_edge(new_vertices[j], new_vertices[j+1], in_g);
	  in_g[e].category = 3;                    
	  in_g[e].non_path_edge = true;
	}
                
	//find old edge
	edge_descriptor e, e_old;
	BGL_FORALL_OUTEDGES_T(vd_paths[i + 1][0], e, in_g, graphType){

	  if (target(e, in_g) == vd_paths[i + 1][1]){
	    e_old = e;
	  }
	}

	//last edge 
	tie (e_new, edge_added) = add_edge(new_vertices[new_vertices.size() - 1], vd_paths[i + 1][1], in_g);
	in_g[e_new].category = in_g[e_old].category;
	in_g[e_new].non_path_edge = in_g[e_old].non_path_edge;
	//cout << "e_new: " << e_new << endl;
                
	//remove old edge
	remove_edge(e_old, in_g);
	            
	vd_paths[i + 1][0] = new_vertices[new_vertices.size() - 1];
    
      }
			       
    }
  }

 private:
	
	
  void attachTwoPaths(int path_index, vector<pathBox>& pathBoxes, pathBox& cur_pathBox){
    //attach pathBoxes
	
    if (path_index == 0)
      return;
	
    pathBox& last_pathBox = pathBoxes[pathBoxes.size() - 2];
    cur_pathBox.setXY(last_pathBox.x + last_pathBox.getLastVertexX(), last_pathBox.y + last_pathBox.getLastVertexY());
    MONOTONICITY m = cur_pathBox.monotonicity;	
    MONOTONICITY m_last = last_pathBox.monotonicity;

    bool threePathLinkEdges = ((m == TOP_TO_BOTTOM) && (m_last == BOTTOM_TO_TOP)) || 
      ((m == BOTTOM_TO_TOP) && (m_last == TOP_TO_BOTTOM)) || 
      ((m == LEFT_TO_RIGHT) && (m_last == RIGHT_TO_LEFT)) || 
      ((m == RIGHT_TO_LEFT) && (m_last == LEFT_TO_RIGHT));
        
    last_pathBox.dPathBox = new dummyPathBox(threePathLinkEdges, m_last, m);
    last_pathBox.dPathBox->setXY(last_pathBox.x + last_pathBox.getLastVertexX(), last_pathBox.y + last_pathBox.getLastVertexY());
            

    //is there a conflict?
    if (!cur_pathBox.intersectsPriorPathBox(last_pathBox))
      return;
        
    if (m_last == LEFT_TO_RIGHT){  
      if (m == TOP_TO_BOTTOM || m == BOTTOM_TO_TOP)
	{ 	                
	  //determine delta_x
	  float delta_x = abs(cur_pathBox.min_x)+ 25;
                			        
	  //move current box by delta_x
	  cur_pathBox.moveBy(delta_x, 0);
	  last_pathBox.dPathBox->increaseWidth(delta_x);
                
	  if ( (cur_pathBox.points[0].y == cur_pathBox.points[1].y) && (cur_pathBox.points[0].x > cur_pathBox.points[1].x) ){
	    double delta_y = (m == TOP_TO_BOTTOM) ? -50 : 50;
	    cur_pathBox.moveBy(0, delta_y);
	    last_pathBox.dPathBox->increaseHeight(delta_y);
            
	  }
	}
      else if ( m == RIGHT_TO_LEFT )
	{
	  //determine delta_y
	  float delta_y_1 = abs(last_pathBox.max_y - last_pathBox.getLastVertexY()) + abs(cur_pathBox.min_y); //nach oben schieben
	  float delta_y_2 = abs(last_pathBox.min_y - last_pathBox.getLastVertexY()) + abs(cur_pathBox.max_y); //nach unten schieben
                		        
	  delta_y_1 += 25;
	  delta_y_2 += 25; //25 = buffer
                

	  if (delta_y_1 < delta_y_2){
	    cur_pathBox.moveBy(0, delta_y_1);
	    last_pathBox.dPathBox->increaseHeight(delta_y_1);	
		            
	    if ( ( (cur_pathBox.points[0].x == cur_pathBox.points[1].x) && (cur_pathBox.points[0].y > cur_pathBox.points[1].y) ) ||
		 ( (last_pathBox.points[last_pathBox.points.size() - 2].x == last_pathBox.points[last_pathBox.points.size() - 1].x) && 
		   (last_pathBox.points[last_pathBox.points.size() - 2].y > last_pathBox.points[last_pathBox.points.size() - 1].y) ) )
	      {
		last_pathBox.dPathBox->increaseWidth(50);
	      }
            
	  }else{
	    cur_pathBox.moveBy(0, -delta_y_2);
	    last_pathBox.dPathBox->increaseHeight(-delta_y_2 );
		            
	    if ( ( (cur_pathBox.points[0].x == cur_pathBox.points[1].x) && (cur_pathBox.points[0].y < cur_pathBox.points[1].y) ) ||
		 ( (last_pathBox.points[last_pathBox.points.size() - 2].x == last_pathBox.points[last_pathBox.points.size() - 1].x) && 
		   (last_pathBox.points[last_pathBox.points.size() - 2].y < last_pathBox.points[last_pathBox.points.size() - 1].y) ) )
	      {
		last_pathBox.dPathBox->increaseWidth(50);
	      }		            			        
	  }
	}
    }
    else if (m_last == RIGHT_TO_LEFT){
      if (m == TOP_TO_BOTTOM || m == BOTTOM_TO_TOP) { 	          
	  //determine delta_x
	  float delta_x = abs(cur_pathBox.max_x)+ 25;
                			        
	  //move current box by delta_x
	  cur_pathBox.moveBy(-delta_x, 0);
	  last_pathBox.dPathBox->increaseWidth(-delta_x);
                
	  if ( (cur_pathBox.points[0].y == cur_pathBox.points[1].y) && (cur_pathBox.points[0].x < cur_pathBox.points[1].x) ){
	    double delta_y = (m == TOP_TO_BOTTOM) ? -50 : 50;
	    cur_pathBox.moveBy(0, delta_y);
	    last_pathBox.dPathBox->increaseHeight(delta_y);
            
	  }
	}	            
        
      else if ( m == LEFT_TO_RIGHT ) {
            
	  //determine delta_y
	  float delta_y_1 = abs(last_pathBox.max_y - last_pathBox.getLastVertexY()) + abs(cur_pathBox.min_y);
	  float delta_y_2 = abs(last_pathBox.min_y - last_pathBox.getLastVertexY()) + abs(cur_pathBox.max_y);
                		        
	  delta_y_1 += 25;
	  delta_y_2 += 25; //25 = buffer

	  if (delta_y_1 < delta_y_2){
	    //nach unten
	    cur_pathBox.moveBy(0, delta_y_1);
	    last_pathBox.dPathBox->increaseHeight(delta_y_1);	
		            
	    if ( ( (cur_pathBox.points[0].x == cur_pathBox.points[1].x) && (cur_pathBox.points[0].y > cur_pathBox.points[1].y) ) ||
		 ( (last_pathBox.points[last_pathBox.points.size() - 2].x == last_pathBox.points[last_pathBox.points.size() - 1].x) && 
		   (last_pathBox.points[last_pathBox.points.size() - 2].y > last_pathBox.points[last_pathBox.points.size() - 1].y) ) )
	      {
		last_pathBox.dPathBox->increaseWidth(-50);
	      }
            
	  }else{
	    //nach oben
	    cur_pathBox.moveBy(0, -delta_y_2);
	    last_pathBox.dPathBox->increaseHeight(-delta_y_2 );
		            
	    if ( ( (cur_pathBox.points[0].x == cur_pathBox.points[1].x) && (cur_pathBox.points[0].y < cur_pathBox.points[1].y) ) ||
		 ( (last_pathBox.points[last_pathBox.points.size() - 2].x == last_pathBox.points[last_pathBox.points.size() - 1].x) && 
		   (last_pathBox.points[last_pathBox.points.size() - 2].y < last_pathBox.points[last_pathBox.points.size() - 1].y) ) )
	      {
		//cur_pathBox.moveBy(-50, 00);
		last_pathBox.dPathBox->increaseWidth(-50);
                    
	      }		            			        
	  }
	}
            
    }
    else if (m_last == BOTTOM_TO_TOP){
      if (m == LEFT_TO_RIGHT || m == RIGHT_TO_LEFT)
	{ 	                
	  //determine delta_x
	  float delta_x = 0;
	  if (m == LEFT_TO_RIGHT)
	    delta_x = (last_pathBox.max_x - last_pathBox.getLastVertexX());
	  else
	    delta_x = -(last_pathBox.getLastVertexX() - last_pathBox.min_x);
                    			        
	  //move current box by delta_x
	  cur_pathBox.moveBy(delta_x, 0);
	  last_pathBox.dPathBox->increaseWidth(delta_x);

	  if (m == RIGHT_TO_LEFT)
	    {
	      if ( ((cur_pathBox.points[0].x < cur_pathBox.points[1].x) && (cur_pathBox.points[0].y == cur_pathBox.points[1].y)) || 
		   ( (last_pathBox.points[last_pathBox.points.size() - 1].x > last_pathBox.points[last_pathBox.points.size() - 2].x) && 
		     (last_pathBox.points[last_pathBox.points.size() - 1].y == last_pathBox.points[last_pathBox.points.size() - 2].y) ) )
		{
		  double delta_y = 50;
		  cur_pathBox.moveBy(0, delta_y);
		  last_pathBox.dPathBox->increaseHeight(delta_y);
		}
	    }else{
	    if ( ((cur_pathBox.points[0].x > cur_pathBox.points[1].x) && (cur_pathBox.points[0].y == cur_pathBox.points[1].y))  ||
		 ( (last_pathBox.points[last_pathBox.points.size() - 1].x < last_pathBox.points[last_pathBox.points.size() - 2].x) && 
		   (last_pathBox.points[last_pathBox.points.size() - 1].y == last_pathBox.points[last_pathBox.points.size() - 2].y) ) )
	      {
		double delta_y = 50;
		cur_pathBox.moveBy(0, delta_y);
		last_pathBox.dPathBox->increaseHeight(delta_y);
	      }
                
	  }
                
	}
      else if (m == TOP_TO_BOTTOM){
            
	//determine delta_y
	float delta_x_1 = last_pathBox.max_x - last_pathBox.getLastVertexX() + abs(cur_pathBox.min_x); //nach rechts
	float delta_x_2 = last_pathBox.getLastVertexX() - last_pathBox.min_x + abs(cur_pathBox.max_x); //nach links
                		        
	delta_x_1 += 25;
	delta_x_2 += 25; //25 = buffer
                

	if (delta_x_1 < delta_x_2){
	  //nach rechts
	  cur_pathBox.moveBy(delta_x_1, 0);
	  last_pathBox.dPathBox->increaseWidth(delta_x_1);	
		            
		            
	  if ( ( (cur_pathBox.points[0].y == cur_pathBox.points[1].y) && (cur_pathBox.points[0].x > cur_pathBox.points[1].x) ) ||
	       ( (last_pathBox.points[last_pathBox.points.size() - 2].y == last_pathBox.points[last_pathBox.points.size() - 1].y) && 
		 (last_pathBox.points[last_pathBox.points.size() - 2].x > last_pathBox.points[last_pathBox.points.size() - 1].x) ) )
	    {
	      last_pathBox.dPathBox->increaseHeight(50);
	    }
		            
            
	}else{
	  //nach links
	  cur_pathBox.moveBy(-delta_x_2, 0);
	  last_pathBox.dPathBox->increaseWidth(-delta_x_2 );
		            

	  if ( ( (cur_pathBox.points[0].y == cur_pathBox.points[1].y) && (cur_pathBox.points[0].x < cur_pathBox.points[1].x) ) ||
	       ( (last_pathBox.points[last_pathBox.points.size() - 2].y == last_pathBox.points[last_pathBox.points.size() - 1].y) && 
		 (last_pathBox.points[last_pathBox.points.size() - 2].x < last_pathBox.points[last_pathBox.points.size() - 1].x) ) )
	    {
	      last_pathBox.dPathBox->increaseHeight(50);
                    
	    }		            			        
                    
	}         
      }
            	
    }
    else if (m_last == TOP_TO_BOTTOM){
      if (m == LEFT_TO_RIGHT || m == RIGHT_TO_LEFT)
	{ 	                
	  //determine delta_x
	  float delta_x = 0;
	  if (m == LEFT_TO_RIGHT)
	    delta_x = (last_pathBox.max_x - last_pathBox.getLastVertexX()) + 25;
	  else
	    delta_x = -((last_pathBox.getLastVertexX() - last_pathBox.min_x)) - 25;
	  //move current box by delta_x
	  cur_pathBox.moveBy(delta_x, 0);
	  last_pathBox.dPathBox->increaseWidth(delta_x);
                
	  //erste Kante nach links
	  if (m == RIGHT_TO_LEFT)
	    {
	      if ( (cur_pathBox.points[0].x < cur_pathBox.points[1].x) && (cur_pathBox.points[0].y == cur_pathBox.points[1].y) ||
		   ( (last_pathBox.points[last_pathBox.points.size() - 1].x  > last_pathBox.points[last_pathBox.points.size() - 2].x) &&
		     (last_pathBox.points[last_pathBox.points.size() - 1].y == last_pathBox.points[last_pathBox.points.size() - 2].y) ) )
		{
		  double delta_y = -50;
		  cur_pathBox.moveBy(0, delta_y);
		  last_pathBox.dPathBox->increaseHeight(delta_y);
		}
	    }else{
	    if ( (cur_pathBox.points[0].x > cur_pathBox.points[1].x) && (cur_pathBox.points[0].y == cur_pathBox.points[1].y) ||
		 ( (last_pathBox.points[last_pathBox.points.size() - 1].x  < last_pathBox.points[last_pathBox.points.size() - 2].x) &&
		   (last_pathBox.points[last_pathBox.points.size() - 1].y == last_pathBox.points[last_pathBox.points.size() - 2].y) ) )
	      {
		double delta_y = -50;
		cur_pathBox.moveBy(0, delta_y);
		last_pathBox.dPathBox->increaseHeight(delta_y);
	      }
                
	  }
                
	}
      else if (m == BOTTOM_TO_TOP){
            
	//determine delta_y
	float delta_x_1 = last_pathBox.max_x - last_pathBox.getLastVertexX() + abs(cur_pathBox.min_x); //nach rechts
	float delta_x_2 = last_pathBox.getLastVertexX() - last_pathBox.min_x + abs(cur_pathBox.max_x); //nach links
                		        
	delta_x_1 += 25;
	delta_x_2 += 25; //25 = buffer
                

	if (delta_x_1 < delta_x_2){
	  //nach rechts
	  cur_pathBox.moveBy(delta_x_1, 0);
	  last_pathBox.dPathBox->increaseWidth(delta_x_1);	
		            
		            
	  if ( ( (cur_pathBox.points[0].y == cur_pathBox.points[1].y) && (cur_pathBox.points[0].x > cur_pathBox.points[1].x) ) ||
	       ( (last_pathBox.points[last_pathBox.points.size() - 2].y == last_pathBox.points[last_pathBox.points.size() - 1].y) && 
		 (last_pathBox.points[last_pathBox.points.size() - 2].x > last_pathBox.points[last_pathBox.points.size() - 1].x) ) )
	    {
	      last_pathBox.dPathBox->increaseHeight(-50);
	    }
		            
            
	}else{
	  //nach links
	  cur_pathBox.moveBy(-delta_x_2, 0);
	  last_pathBox.dPathBox->increaseWidth(-delta_x_2 );
		            

	  if ( ( (cur_pathBox.points[0].y == cur_pathBox.points[1].y) && (cur_pathBox.points[0].x < cur_pathBox.points[1].x) ) ||
	       ( (last_pathBox.points[last_pathBox.points.size() - 2].y == last_pathBox.points[last_pathBox.points.size() - 1].y) && 
		 (last_pathBox.points[last_pathBox.points.size() - 2].x < last_pathBox.points[last_pathBox.points.size() - 1].x) ) )
	    {
	      last_pathBox.dPathBox->increaseHeight(-50);
                    
	    }		            			        
                    
	}         
                
      }
            
            	
    }

    nmb_added_edges = 0;
    for (int i = 0; i < pathBoxes.size() - 1; i++){
      dummyPathBox d = *pathBoxes[i].dPathBox;
            
      for (int j = 0; j < d.points.size() - 1; j++){
	if (d.points[j].x != d.points[j + 1].x ||
	    d.points[j].y != d.points[j + 1].y)
	  nmb_added_edges++;
      }
        
    }
  }

  MONOTONICITY getMonotonicity(int path_index) {
    int monotonicity = 1 | 1 << 1 | 1 << 2 | 1 << 3;
    for (size_t i = 0; i < paths_schem[path_index].size() - 1; i++) {
      if (paths_schem[path_index][i + 1].x < paths_schem[path_index][i].x) {
	monotonicity &=  0 | 1 << 1 | 1 << 2 | 1 << 3;
      }
      else if (paths_schem[path_index][i + 1].x > paths_schem[path_index][i].x) {
	monotonicity &=  1 | 0 << 1 | 1 << 2 | 1 << 3;
      }

      if (paths_schem[path_index][i + 1].y < paths_schem[path_index][i].y) {
	monotonicity &=  1 | 1 << 1 | 0 << 2 | 1 << 3;
      }
      else if (paths_schem[path_index][i + 1].y > paths_schem[path_index][i].y) {
	monotonicity &=  1 | 1 << 1 | 1 << 2 | 0 << 3;
      }
    }
       
    assert(monotonicity != 0);

    if ((monotonicity & 1) == 1){
      return LEFT_TO_RIGHT;
    }
    else if ((monotonicity & (1 << 1)) == (1 << 1) ){
      return RIGHT_TO_LEFT;
    }
    else if ((monotonicity & (1 << 2)) == (1 << 2) ){
      return BOTTOM_TO_TOP;
    }
    else if ((monotonicity & (1 << 3)) == (1 << 3) ){
      return TOP_TO_BOTTOM;
    }
			
    assert(false);
		
  }
	

 private:
		
  vector< vector<schemPoint> > paths;
  vector< vector<schemPoint> > paths_schem;
  vector< vector<vertex_descriptor> > vd_paths;
  vector< vector<vertex_descriptor> > vd_paths_stats;
  int nmb_added_edges;
  int nmb_edges,	nmb_out_edges, nmb_pairs_oo_preserved,  nmb_pairs_oo_violated;

};
#endif
