/******************************************************************************
* graph.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * graph.h
 *
 *  Created on: Jun 30, 2010
 *      Author: pajor
 */

#ifndef GRAPH_H_
#define GRAPH_H_


#include "../geometry/point.h"
#include "../geometry/line.h"
#include "../geometry/tools.h"
#include "../geometry/latLng.h"

#include <string>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/graph_traits.hpp>

using namespace boost;
#define DUMMY_NODE_ID 98765432

namespace schematization {


struct node_properties {
	int node_id;

	point coord;
	// This indicates, whether a node should always be kept.
	bool important;


	// Set some default values here
	node_properties() : coord(0.0, 0.0), important(true) {}
};

 struct edge_properties {
   int category; // The depicted street category
   int edge_id;
   bool non_path_edge; //only used for the heuristic
  
   string name = "dummyname";
   // Set some default values here
 edge_properties() :  category(0), edge_id(-1), non_path_edge(false) {}
 };



// This is the graph type
typedef adjacency_list<vecS, // vector for edges
					   vecS, // vector for nodes
                       bidirectionalS, // we need access to in-edges
                       node_properties,
                       edge_properties> schemGraph;

// Returns the first incoming edge for a node v (there must exist an edge)
template<typename graphType>
typename graphType::edge_descriptor get_first_in_edge(typename graphType::vertex_descriptor &v, graphType &graph) {
	typedef typename graph_traits<graphType>::in_edge_iterator in_edge_iterator;

	in_edge_iterator first_e, last_e;
	tie(first_e, last_e) = in_edges(v, graph);

	return *first_e;
}


// Returns the first outgoing edge for a node v (there must exist an edge)
template<typename graphType>
typename graphType::edge_descriptor get_first_out_edge(typename graphType::vertex_descriptor &v, graphType &graph) {
	typedef typename graph_traits<graphType>::out_edge_iterator out_edge_iterator;

	out_edge_iterator first_e, last_e;
	tie(first_e, last_e) = out_edges(v, graph);

	return *first_e;
}


// This method tests whether two edges are adjacent
template<typename graphType>
bool is_adjacent(const typename graphType::edge_descriptor &e1, const typename graphType::edge_descriptor &e2, const graphType &graph) {
	const typename graphType::vertex_descriptor u1(source(e1,graph)), v1(target(e1,graph)), u2(source(e2,graph)), v2(target(e2,graph));
	return (u1==u2 || v1==u2 || u1==v2 || v1 == v2);
}


// This method retrieves the angle of an edge e
template<typename graphType>
double get_edge_angle(const typename graphType::edge_descriptor &e, const graphType &graph) {
	return tools::get_angle(get_edge_line(e, graph));
}


// This method retrieves the quadrant of an edge e
template<typename graphType>
int get_edge_quadrant(const typename graphType::edge_descriptor &e, const graphType &graph) {
	return tools::get_quadrant(get_edge_line(e, graph));
}

// This method retrieves the quadrant of an edge e
template<typename graphType>
int get_edge_sector(const typename graphType::edge_descriptor &e, const angleType a, const graphType &graph) {
	return tools::get_sector(get_edge_line(e, graph), a);
}


// This method retrieves the line represented by an edge of the graph
template<typename graphType>
line get_edge_line(const typename graphType::edge_descriptor &e, const graphType &graph) {
	const typename graphType::vertex_descriptor u = source(e, graph);
	const typename graphType::vertex_descriptor v = target(e, graph);

	return line(graph[u].coord, graph[v].coord);
}


// This returns the bounding box of a graph
template<typename graphType>
boundingBox get_graph_bounding_box(const graphType &graph) {
	// If the graph is empty, return bb at 0,0
	if (num_vertices(graph) == 0)
		return boundingBox(point(0.0,0.0));

	typedef typename graph_traits<graphType>::vertex_iterator vertex_iterator;
	vertex_iterator current_v, last_v;
	tie(current_v, last_v) = vertices(graph);
	boundingBox bb(graph[*current_v].coord);
	for (++current_v; current_v != last_v; ++current_v) {
		bb.extend(graph[*current_v].coord);
	}
	return bb;
}




}

#endif /* GRAPH_H_ */
