/******************************************************************************
* schemStats.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * schemStats.h
 *
 *  Created on: May 18, 2010
 *      Author: pajor
 */

#ifndef SCHEMSTATS_H_
#define SCHEMSTATS_H_

#include "gurobi_c++.h"

#include "types.h"
#include "dataStructures/graph.h"
#include "milp/schemMIP.h"
#include "axismonotone/schemePathWrapper.h"

#include "time.h"

#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/path.hpp"
#include <boost/graph/iteration_macros.hpp>

#include "io/graphIO.h"

#include <fstream>
#include <algorithm>

using namespace std;

namespace schematization {

  class schemStats {
    typedef schematization::schemGraph schemGraphType;

  public:

    // Construct the class by passing several important data
  schemStats(const angleType angl, const double e) :
    filenames(0),
      graphs(0),
      eps(e),
      angle(angl)
      {
	// Disable gurobi output
	gurobi_env.set(GRB_IntParam_OutputFlag, 1);
	gurobi_env.set(GRB_IntParam_Threads, 1);
      }

    void load_directory_nodefiles(const string &directory) {
      namespace fs = boost::filesystem;

      fs::path full_path = fs::system_complete(fs::path(directory));

      // Check if directory exists
      if (!fs::exists(full_path))	{
	cout << "Directory " << full_path.string() << " not found." << endl;
	return;
      }

      // Check if directory is indeed a directory
      if (!fs::is_directory(full_path)) {
	cout << "Error: " << full_path.string() << " is not a directory." << endl;
	return;
      }

      // Iterate over the directory
      fs::directory_iterator end_iter;
      cout << "Reading node id files from directory... " << flush;
      size_t num_files(0);
		
      for (fs::directory_iterator dir_itr(full_path); dir_itr != end_iter; ++dir_itr)	{
	if (fs::is_regular_file(dir_itr->status())) {
	  string full_filename(fs::system_complete(dir_itr->path()).string());
	  filenames.push_back(full_filename);
	}
      }
	
      std::sort(filenames.begin(), filenames.end());
      
      
      //read graphs
      for (size_t i = 0; i < filenames.size(); i++) {
	string full_filename = filenames[i];

	schemGraphType g;
	read_graph<schemGraphType>(g, full_filename);

	graphs.push_back(g);
	++num_files;			
      }
		
      cout << "done (" << num_files << ")" << endl;
    }


    // Schematization
    void schematize() {
      cout << "Preparing... " << flush;
      simplified_graphs.resize(graphs.size());
      schematized_graphs.resize(graphs.size());
      running_time.resize(graphs.size());
      infeasible.resize(graphs.size());
      simple.resize(graphs.size());
      num_iterations.resize(graphs.size());
      cout << "done." << endl;

      cout << "Schematizing paths... " << endl;
      for (size_t i = 0; i < graphs.size(); ++i) {
	cout << "[" << i << "]: " << filenames[i] << "... " << endl;
	schematize_path(i);
      }
      cout << endl << "done." << endl;
    }
	
    // Schematization
    void schematizeAxsisMonotone() {
      cout << "Preparing... " << flush;
      simplified_graphs.resize(graphs.size());
      schematized_graphs.resize(graphs.size());
      running_time.resize(graphs.size());
      cout << "done." << endl;

      cout << "Schematizing paths using axismonotone algo... " << endl;
      for (size_t i = 0; i < graphs.size(); ++i) {
	cout << "[" << i << "]: " << filenames[i] << "... " << endl;
	schematize_path_axismonotone(i);
      }
      cout << endl << "done." << endl;
    }

    // Dump some stats
    void dump_stats(ostream &os, ostream &osf) {
      // Accumulate data
      const size_t num_paths = graphs.size();
      double sum_running_time(0.0), f_sum_running_time(0.0);
      double max_running_time(0.0), f_max_running_time(0.0);
      size_t sum_infeasible(0), f_sum_infeasible(0);
      size_t sum_nonsimple(0), f_sum_nonsimple(0);
      size_t sum_num_iterations(0), f_sum_num_iterations(0);
      size_t sum_path_lengths(0), f_sum_path_lengths(0);
      size_t sum_schematized_path_lengths(0), f_sum_schematized_path_lengths(0);
      size_t f_num_paths(0);
      for (size_t i = 0; i < num_paths; ++i) {
	sum_path_lengths += num_edges(graphs[i]);
	sum_schematized_path_lengths += num_edges(simplified_graphs[i]);
	sum_infeasible += infeasible[i];
	sum_nonsimple += !simple[i];
	sum_num_iterations += num_iterations[i];
	sum_running_time += running_time[i];
	max_running_time = std::max(max_running_time, running_time[i]);
	if (!infeasible[i]) {
	  f_sum_path_lengths += num_edges(graphs[i]);
	  f_sum_schematized_path_lengths += num_edges(simplified_graphs[i]);
	  f_sum_infeasible += infeasible[i];
	  f_sum_nonsimple += !simple[i];
	  f_sum_num_iterations += num_iterations[i];
	  f_sum_running_time += running_time[i];
	  f_max_running_time = std::max(f_max_running_time, running_time[i]);
	  ++f_num_paths;
	}
      }

      double mean_running_time   = sum_running_time / double(num_paths);
      double infeasible_perc     = double(sum_infeasible) / double(num_paths) * 100.0;
      double nonsimple_perc      = double(sum_nonsimple) / double(num_paths) * 100.0;
      double mean_num_iterations = double(sum_num_iterations) / double(num_paths);
      double mean_path_lengths   = double(sum_path_lengths) / double(num_paths);
      double mean_schematized_path_lengths = double(sum_schematized_path_lengths) / double(num_paths);

      double f_mean_running_time   = f_sum_running_time / double(f_num_paths);
      double f_infeasible_perc     = double(f_sum_infeasible) / double(f_num_paths) * 100.0;
      double f_nonsimple_perc      = double(f_sum_nonsimple) / double(num_paths) * 100.0;
      double f_mean_num_iterations = double(f_sum_num_iterations) / double(f_num_paths);
      double f_mean_path_lengths   = double(f_sum_path_lengths) / double(f_num_paths);
      double f_mean_schematized_path_lengths = double(f_sum_schematized_path_lengths) / double(f_num_paths);

      // The header
      os   << endl
	   << setw(5)  << " "
	   << " | " << setw(7) << right << "# paths"
	   << " | " << setw(7) << right << "~in len"
	   << " | " << setw(8) << right << "~out len"
	   << " | " << setw(6) << right << "%inf."
	   << " | " << setw(6) << right << "%nsimp."
	   << " | " << setw(6) << right << "~iter."
	   << " | " << setw(10) << right << "~time [ms]"
	   << " | " << setw(10) << right << "^time [ms]"
	   << endl
	   << "-----------------------------------------------------------------------------------------"
	   << endl
	   << setw(5)  << "all"
	   << " | " << setw(7) << right << num_paths
	   << " | " << setw(7) << fixed << setprecision(1) << mean_path_lengths
	   << " | " << setw(8) << fixed << setprecision(1) << mean_schematized_path_lengths
	   << " | " << setw(6) << fixed << setprecision(2) << infeasible_perc
	   << " | " << setw(6) << fixed << setprecision(2) << nonsimple_perc
	   << " | " << setw(6) << fixed << setprecision(2) << mean_num_iterations
	   << " | " << setw(10) << fixed << setprecision(2) << mean_running_time
	   << " | " << setw(10) << fixed << setprecision(2) << max_running_time
	   << endl
	   << setw(5)  << "feas."
	   << " | " << setw(7) << right << f_num_paths
	   << " | " << setw(7) << fixed << setprecision(1) << f_mean_path_lengths
	   << " | " << setw(8) << fixed << setprecision(1) << f_mean_schematized_path_lengths
	   << " | " << setw(6) << fixed << setprecision(2) << f_infeasible_perc
	   << " | " << setw(6) << fixed << setprecision(2) << f_nonsimple_perc
	   << " | " << setw(6) << fixed << setprecision(2) << f_mean_num_iterations
	   << " | " << setw(10) << fixed << setprecision(2) << f_mean_running_time
	   << " | " << setw(10) << fixed << setprecision(2) << f_max_running_time
	   << endl << endl;

      if  (osf != os) {
	osf << setw(7) << eps
	    << " & " << setw(5) << angle
	    << " & " << setw(7) << fixed << setprecision(1) << mean_path_lengths
	    << " & " << setw(8) << fixed << setprecision(1) << mean_schematized_path_lengths
	    << " & " << setw(6) << fixed << setprecision(2) << infeasible_perc
	    << " & " << setw(6) << fixed << setprecision(2) << nonsimple_perc
	    << " & " << setw(6) << fixed << setprecision(2) << mean_num_iterations
	    << " & " << setw(10) << fixed << setprecision(2) << mean_running_time
	    << "\\\\"  << endl;
      }
    }

    // Dump some stats
    void dump_stats_axismonotone(ostream &os, ostream &osf) {
      // Accumulate data
      const size_t num_paths = graphs.size();
      double sum_running_time(0.0);
      double max_running_time(0.0);
      size_t sum_path_lengths(0);
      size_t sum_schematized_path_lengths(0);
      for (size_t i = 0; i < num_paths; ++i) {
	sum_path_lengths += num_edges(graphs[i]);
	sum_schematized_path_lengths += num_edges(simplified_graphs[i]);
	sum_running_time += running_time[i];
	max_running_time = std::max(max_running_time, running_time[i]);
      }

      double mean_running_time   = sum_running_time / double(num_paths);
      double mean_path_lengths   = double(sum_path_lengths) / double(num_paths);
      double mean_schematized_path_lengths = double(sum_schematized_path_lengths) / double(num_paths);


      // The header
      os   << endl
	   << setw(5)  << " "
	   << " | " << setw(7) << right << "# paths"
	   << " | " << setw(7) << right << "~in len"
	   << " | " << setw(8) << right << "~out len"
	   << " | " << setw(6) << right << "%inf."
	   << " | " << setw(6) << right << "%nsimp."
	   << " | " << setw(6) << right << "~iter."
	   << " | " << setw(10) << right << "~time [ms]"
	   << " | " << setw(10) << right << "^time [ms]"
	   << endl
	   << "-----------------------------------------------------------------------------------------"
	   << endl
	   << setw(5)  << "all"
	   << " | " << setw(7) << right << num_paths
	   << " | " << setw(7) << fixed << setprecision(1) << mean_path_lengths
	   << " | " << setw(8) << fixed << setprecision(1) << mean_schematized_path_lengths
	   << " | " << setw(10) << fixed << setprecision(2) << mean_running_time
	   << " | " << setw(10) << fixed << setprecision(2) << max_running_time
	   << endl << endl;

      if  (osf != os) {
	osf << setw(7) << eps
	    << " & " << setw(5) << angle
	    << " & " << setw(7) << fixed << setprecision(1) << mean_path_lengths
	    << " & " << setw(8) << fixed << setprecision(1) << mean_schematized_path_lengths
	    << " & " << setw(10) << fixed << setprecision(2) << mean_running_time
	    << "\\\\"  << endl;
      }
    }

    // Dumps a raw csv file, separated by ;
    inline bool dump_csv(const string &filename) const {
      cout << "Dumping csv-file to " << filename << "... " << flush;

      // Try to open the file
      ofstream f(filename.c_str());
      if (!f.is_open()) {
	cout << "failed." << endl;
	return false;
      }

      // Dump header.
      f << "eps;angle;in_length;out_length;inf;nonsimple;iterations;time;filename" << endl;

      for (size_t i = 0; i < graphs.size(); ++i) {
	f << eps
	  << ";" << angle
	  << ";" << num_edges(graphs[i])
	  << ";" << num_edges(simplified_graphs[i])
	  << ";" << (infeasible[i] ? "1" : "0")
	  << ";" << (simple[i] ? "0" : "1")
	  << ";" << num_iterations[i]
	  << ";" << running_time[i]
	  << ";" << filenames[i]
	  << endl;
      }

      f.close();
      cout << "done." << endl;
      return true;
    }

    // Dumps a raw csv file, separated by ;
    inline bool dump_csv_axismonotone(const string &filename) const {
      cout << "Dumping csv-file to " << filename << "... " << flush;

      // Try to open the file
      ofstream f(filename.c_str());
      if (!f.is_open()) {
	cout << "failed." << endl;
	return false;
      }

      // Dump header.
      f << "eps;angle;in_length;out_length;added_edges;nmb_out_edges;nmb_paths;pairs_oo_preserved;pairs_oo_violated;nmb_pairs;pairs_preserved_percentage;time;filename" << endl;

      for (size_t i = 0; i < graphs.size(); ++i) {
	f << eps
	  << ";" << angle
	  << ";" << num_edges(graphs[i])
	  << ";" << num_edges(simplified_graphs[i])
	  << ";" << axismonotone_stats[i].nmb_added_edges 
	  << ";" << axismonotone_stats[i].nmb_out_edges
	  << ";" << axismonotone_stats[i].nmb_paths 
	  << ";" << axismonotone_stats[i].nmb_pairs_oo_preserved 
	  << ";" << axismonotone_stats[i].nmb_pairs_oo_violated 
	  << ";" << axismonotone_stats[i].nmb_pairs_oo_preserved + axismonotone_stats[i].nmb_pairs_oo_violated 
	  << ";" << (float)axismonotone_stats[i].nmb_pairs_oo_preserved / (float)(axismonotone_stats[i].nmb_pairs_oo_preserved + axismonotone_stats[i].nmb_pairs_oo_violated) * 100.0
	  << ";" << running_time[i]
	  << ";" << filenames[i]
	  << endl;
      }

      f.close();
      cout << "done." << endl;
      return true;
    }



  protected:

    // Schematize a path and measure performance
    bool schematize_path(const size_t i) {
      simplify_graph(graphs[i], simplified_graphs[i], eps);	
      schemMIP<schemGraphType> model(gurobi_env, simplified_graphs[i], angle);
      //      model.disable_output();

      Timer t;
      model.solve(schematized_graphs[i]);
      running_time[i] = t.elapsed();
      infeasible[i] = !model. is_optimal();
      num_iterations[i] = model.get_num_iterations();

      return !model.is_infeasible();
    }

    // Schematize a path and measure performance
    bool schematize_path_axismonotone(const size_t i) {
      simplify_graph(graphs[i], simplified_graphs[i], eps);
		
      Timer t;
		
      schemePathWrapper<schemGraphType> spw;

      schemGraphType schematized_graph;
      spw.schematize(simplified_graphs[i], schematized_graph, angle, false);
      Axismonotone_stats s;
      spw.getStatistics(s);
      running_time[i] = t.elapsed();
      axismonotone_stats.push_back(s);
		
      return true;
    }
	
  private:
    // This is the gurobi opt. environment
    GRBEnv gurobi_env;


    // This holds the filenames (if read from directory)
    vector<string> filenames;

    // This is a vector of graphs to schematize
    vector<schemGraphType> graphs;

    // This is the vector of paths to be schematized and several other statistics
    vector<schemGraphType> simplified_graphs;
    vector<schemGraphType> schematized_graphs;
    vector<double> running_time;
    vector<bool> simple;
    vector<bool> infeasible;
    vector<size_t> num_iterations;
    vector<Axismonotone_stats> axismonotone_stats;

    // Douglas Peucker Eps
    const double eps;

    // Schematization angle
    const angleType angle;

    // Constants
    const static unsigned int col_width = 10;

  };

}

#endif /* SCHEMSTATS_H_ */
